##------------------------------------------------------------------------------
## GUTS parallel example script
## Author: B. Goussen
## Date: 2015/05/15
## GUTS 1.0.0
##------------------------------------------------------------------------------

## Initialisation and package loading
##------------------------------------------------------------------------------
rm(list=ls()) # Clear all objects
graphics.off() # Close all graphics

library('gplots')    # Package for plots
library('lattice')   # Implementation of Trellis graphics for R
library('coda')      # Analysis of MCMC chains
library('GUTS')      # General Unified Theory of Survival implementation for R
library('adaptMCMC') # Monte Carlo Markov chain sampler

# Check for last package version
if( packageVersion('GUTS') < "1.0.0" )
    stop("Update GUTS package to version > 1.0.0")


guts_model <- 'proper' # Define GUTS model (proper or IT)
guts_distribution <- 'lognormal' # GUTS distribution
pars_names <- c( 'hb', 'kr', 'kk', 'm', 'sd' ) # GUTS parameter names
pars_start <- c(0.005, 0.1, 10, 1, 1e-3) # Parameters initial values


## Functions definition
##------------------------------------------------------------------------------
# Define logprior and logposterior

# Logprior always returns 0 if prior is not implemented
logprior <- function( par ) {
    out <- 0
    return (out)
}

# Logposterior definition. par: parameters
# Guts objects are in a list called guts_objects
logposterior <- function(guts_objects, par) {
    out <- 0
    invisible( lapply( guts_objects, function(x) guts_calc_loglikelihood(x,par=par) ) )
    out <- out + sum(sapply( guts_objects, function(x) x$LL ))
    #out <- out + logprior( par ) # if logprior usable, use this additional line
    return(out)
}

## Data loading
##------------------------------------------------------------------------------
data <- read.table('file') 

# Data Example
# C    y0   y1  y2 y3 y4 y5 y6 y7 y8 y9 y10
# 0.00 100 100 100 98 98 97 97 97 96 95  95
# 0.35 100  99  99 97 97 97 95 95 95 95  95
# 0.54 100 100  98 97 95 95 95 88 87 84  82
# 0.74 100 100 100 89 86 77 74 72 70 63  58
# 1.10 100  96  85 70 62 49 40 35 31 28  26
# 1.90 100  95  72 46 28 18 11  3  2  0   0
# 2.80 100  77  45 20 11  5  2  1  1  0   0
# 3.80 100  81  35  7  3  2  1  1  0  0   0


C <- data$C # Load concentration vector
Ct <- ncol(data) - 1 # Load concentration time vector
y <- data[,-1] # Load survival vector
yt <- ncol(data) - 1 # Load survival time vector


## Create GUTS objects
##------------------------------------------------------------------------------

guts_objects <- list()
for ( i in 1:nrow(data) ) {
    # create objects
    guts_objects[[i]] <- guts_setup(C = rep(C[i], Ct),
                                    Ct = 0:(Ct-1), # Need to start from 0
                                    y = as.numeric(y[i,]),
                                    yt = 0:(yt-1), # Need to start from 0
                                    #par = 0,
                                    dist = guts_distribution,
                                    model = guts_model,
                                    N = 1000,
                                    M = 10000)
    
}

## Parallel MCMC parameter estimation
##------------------------------------------------------------------------------
sigma <- diag( rep( 0.01, 5 )) 
sigma <- as.matrix( nearPD(sigma)$mat )

strt<-Sys.time() #start time

# Run 3 parallel MCMC chains
mcmc_results <- MCMC.parallel(
    p        = logposterior,
    init     = pars_start,
    scale    = sigma,
    adapt    = 50000,
    acc.rate = 0.2,
    n        = 200000,
    packages = c('GUTS','adaptMCMC'),
    n.chain = 3,
    n.cpu = 3,
    guts_objects = guts_objects)


print(Sys.time()-strt) # Print time to compute MCMC chains


## Coda analysis of convergence
##------------------------------------------------------------------------------
mcmc_coda <- convert.to.coda(mcmc_results) # Convert to a coda object
gelman <- gelman.diag(mcmc_coda); gelman # Gelman convergence diagnostic

if(max(gelman$psrf[,1]) > 1.05){
    warning('Chains have not converged to an acceptable solution')}


# Plots of the gelman diagnostic, trace of chains, and posterior distribution
gelman.plot(mcmc_coda)
traceplot(mcmc_coda)
densityplot(mcmc_coda)


## Best fit for chain 1
##------------------------------------------------------------------------------
best_fit_pos <- which.max(mcmc_results[[1]]$log.p)
pars_best_fit <- mcmc_results[[1]]$samples[best_fit_pos, ]
LL_best_fit <- mcmc_results[[1]]$log.p[best_fit_pos]

# Print outputs for chain 1
print(paste("Best fit for chain 1, log-likelihood =", LL_best_fit))
print(paste(pars_names, signif(pars_best_fit,4)))
