% This script creates confidence intervals on model curves from the MCMC
% sample that was saved by calc_slice.m, or from the LHS sample that was
% saved by calc_likregion.m. This script only calculates the bounds; they
% are plotted by a new call to calc_and_plot.m.
%
% Author : Tjalling Jager 
% Date: Aug 2015
% Web support: http://www.debtox.info/byom.html

if exist('type_conf','var') == 0 % if the variable type_conf does not exist ...
    type_conf = 1; % by default, assumes there is an MCMC sample
end

%% Select the correct file to load, depending on the required analysis

switch type_conf
    case 1 % sample from posterior distribution, take 95% of MODEL CURVES
        if exist([mfilename_base,'_MC.mat']) ~= 2
            error('There is no MCMC sample saved, so run calc_slice first!')
        end
        load([mfilename_base,'_MC']) % load the random sample from the last MCMC run
        % this loads the random sample in rnd, and the parameter matrix par and the
        % selection matrix par_sel
        
    case 2 % LHS sample from joint conf. region, take min and max of curves
        if exist([mfilename_base,'_LR.mat']) ~= 2
            error('There is no confidence set saved, so run calc_likregion first!')
        end
        load([mfilename_base,'_LR']) % load the random sample from the last calc_likregion run
        % this loads the sets from the conf region in rnd, and the parameter matrix par and the
        % selection matrix par_sel
        rnd = acc; % use only the ACCEPTED sample to calculate the confidence bounds
        
    case 3 % ACCEPTED sample from posterior distribution (highest 95% posterior density), take min and max on curves
        if exist([mfilename_base,'_MC.mat']) ~= 2
            error('There is no MCMC sample saved, so run calc_slice first!')
        end
        load([mfilename_base,'_MC']) % load the random sample from the last MCMC run
        % this loads the random sample in rnd, and the parameter matrix par and the
        % selection matrix par_sel
        rnd = acc; % use only the ACCEPTED sample to calculate the confidence bounds
end

%% Calculating model curves for each set in the sample

disp(' ')
disp('Calculating confidence intervals on model curves ... please be patient.')
drawnow % empty plot buffer if there's something in it
n_samples = size(rnd,1); % number of MCMC samples or sets from the conf region

% initialise the matrices that will collect the output (time x scenario x samples)
for i = 1:n_X % run through state variables
    X2{i} = zeros(length(t),n_s,n_samples); % initialise state variable output with zeros
end

% this is for the progress displayer
progr = [0 10 20 30 40 50 60 70 80 90 100; 0 0 0 0 0 0 0 0 0 0 0];
progr_last = 0;
fprintf('Percentage of samples finished: ')

% par_rem = par_out; % make a copy of the parameter vector
pmat   = packunpack(1,par,0);  % transform structure into a regular matrix
pfit   = pmat(pmat(:,2)==1,1); % parameter values that are to be estimated

for k = 1:n_samples, % run through all samples
    
    pmat(pmat(:,2)==1,1) = rnd(k,:); % replace values in pars with the k-th random sample from the MCMC
    % put parameters that need to be fitted on log scale back on normal
    % scale (as call_deri requires normal scale, in contrast to transfer.m)
    ind_logfit = (pmat(:,5)==0 & pmat(:,2)==1); % indext to pars on log scale that are also fitted!
    if sum(ind_logfit)>0
        pmat(ind_logfit,1) = 10.^(pmat(ind_logfit,1));
    end
    % Note: pmat is on normal scale here, but the sample in rnd contains
    % the value on a log scale, if a parameter is fitted on log scale.
    
    par_k = packunpack(2,0,pmat); % transform parameter matrix into a structure
    
    for j = 1:n_s % run through our scenarios
        
        [Xout TE] = call_deri(t,par_k,X0mat(:,j)); % use call_deri.m to provide the output for one scenario
        
        for i = 1:n_X  % run through state variables
            X2{i}(:,j,k) = Xout(:,i); % collect state variable i into structure X2
        end
        
    end
    
    % display progress in intervals given in progr
    if  100*(k/n_samples)> progr_last
        fprintf('%4.0f',progr_last)
        progr_last = progr(1,find(progr(1,:)==progr_last)+1);
        progr(2,progr(1,:)==progr_last) = 1;
    end
end
fprintf('\n') % end the line opened by the progress monitor

%% TEST! Calculate and plot sensitivity of parameters over time

state_sel = 1; % only use ONE state variable for the moment, set to zero to skip

if state_sel > 0
    
    ind_rnd = find(pmat(:,2)==1); % index in pmat for the parameters in the sample rnd
    corr_coll = nans(length(t),length(ind_rnd),n_s);
    for k = 1:n_s % run through scenarios
        for i = 1:length(t) % run through time points
            for j = 1:length(ind_rnd) % run through parameters in the sample
                tmp = corrcoef(X2{state_sel}(i,k,:),rnd(:,j)); % corr. parameter value and state
                % tmp = corrcoef(X2{state_sel}(i,k,:)./X2{state_sel}(i,1,:),rnd(:,j));% corr. parameter value and effect relative to state 1
                
%                 if i < length(t) % we look at change, so ignore last time point
%                     rel_state = (X2{state_sel}(i+1,k,:)-X2{state_sel}(i,k,:))./X2{state_sel}(i,k,:);
%                     tmp = corrcoef(rel_state,rnd(:,j));% corr. parameter value and effect relative to state 1
%                 else
%                     tmp = [NaN NaN; NaN NaN];
%                 end
                
                corr_coll(i,j,k) = tmp(2);
            end
        end
    end
    
    plotcol = 'krbgcm'; % 6 colors should be enough ...
    
    % make one plot with subplots for each scenario (i.e., concentration)
    n = ceil(sqrt(n_s));
    m = ceil(n_s/n);
    
    figure
    
    for k = 1:n_s % run through scenarios
        subplot(m,n,k)
        title([leglab1,num2str(X0mat(1,k)),leglab2])
        hold on
        
        for j = 1:length(ind_rnd) % run through parameters in the sample
            if j <= 6
                plot(t,corr_coll(:,j,k),[plotcol(j),'-'],'LineWidth',2)
            else % for more than 6 parameters, use dotted line
                plot(t,corr_coll(:,j,k),[plotcol(j-6),':'],'LineWidth',2)
            end
        end
        xlabel(xlab,'FontSize',12)
        ylabel(['correlation to state ',num2str(state_sel)],'FontSize',12)
        % ylabel(['correlation to state/control ',num2str(state_sel)],'FontSize',12)
        % ylabel(['correlation to rel. change of state ',num2str(state_sel)],'FontSize',12)
        legend(names{ind_rnd})
        set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting
        plot([t(1) t(end)],[0 0 ],'k:','LineWidth',2)
    end
    error
end

%% Calculating intervals on the model curves

switch type_conf
    case 1
        for i = 1:n_X % run through state variables
            Xlo{i} = prctile(X2{i},2.5,3);  % 2.5 percentile of model lines
            Xhi{i} = prctile(X2{i},97.5,3); % 97.5 percentile of model lines
        end
    case 2
        for i = 1:n_X % run through state variables
            Xlo{i} = min(X2{i},[],3); % minimum of model lines
            Xhi{i} = max(X2{i},[],3); % maximum of model lines
        end
    case 3
        for i = 1:n_X % run through state variables
            Xlo{i} = min(X2{i},[],3); % minimum of model lines
            Xhi{i} = max(X2{i},[],3); % maximum of model lines
        end
end
    
disp(['Time required: ' secs2hms(toc)])
calc_and_plot; % again, call the script which calculates and plots
