function [confD,confU] = calc_proflik(par,parname,varargin)

% Calculates a profile likelihood for parameter with the name <parname>
% from your parameter structure. <par> is the total parameter structure, 
% resulting from the optimisation. The varargin can be used to specify the
% level of detail of the profiling, detailed (1) or course (2). The default
% is 'course'.
%
% This function is slightly modified from the function in DEBtoxM.
%
% This function applies a variable stepsize algorithm. When the likelihood
% ratio changes very little (less than Lcrit_min), the stepsize is
% increased (up to a maximum, specified by Fstep_max). When the lik. ratio
% changes too much (more than Lcrit_max), the algorithm tries again with a
% smaller stepsize (also bound to a minimum: Fstep_min). Note that the
% stepsize is used as a fraction of the parameter value that is tried. To
% prevent very small stepsizes when the value goes towards zero (as can be
% the case for NECs), I also apply an *absolute* minimum stepsize
% (Fstep_abs), which is specified as a fraction of the best parameter value
% (Xhat) (unless it is zero, then algoritm takes something small!).
%
% Author : Tjalling Jager
% Date: Aug 2015
% Web support: http://www.debtox.info/byom.html
 
global names ttot
% Note, the profiles require ttot and ctot to be defined as globals, which
% is done in prelim_checks.m. These parameters are required by transfer.m.
% Only the names parameter is used here for plotting and reporting.

if isempty(names) % if this script is run without refitting, 'names' is  not defined
    names  = fieldnames(par); % extract all field names of par (global)
end % this is probably not needed anymore ...

% level of detail of the likelihood profiles: 1 is detailed, 2 is course (but rapid)
if isempty(varargin)
    prof_detail = 2; % if no value is entered, 'course' is default
    % this is done to ensure compatibility with older BYOM scripts
else
    prof_detail = varargin{1};
    if  ~ismember(prof_detail,[1 2])
        prof_detail = 2; % if a wrong value is entered, 'course' is default
    end
end

parnum = find(strcmp(names,parname)); % lookup the name in the structure and return the position
if isempty(parnum)
    error(['The name of the parameter to profile (',parname,') does not exist in your structure.'])
end
if exist('ttot') == 0 % see if the global ttot is available
    error('Run an optimisation at least once before profiling the likelihood.')
end
% tic % turn on the stopwatch

fid = fopen('profiles_newopt.out','w'); % file to save all new optima as soon as the profile finds some
% the 'w' option destroys the old contents of the file; use 'a' to append 

pmat    = packunpack(1,par,0);  % transform structure into a regular matrix
% put parameters that need to be on log scale on log10 scale
pmat(pmat(:,5)==0,1) = log10(pmat(pmat(:,5)==0,1));
% also modify the min max ranges!
pmat2 = pmat; % but do this on a copy, as transfer wants the non-log version
pmat2(pmat2(:,5)==0,[3 4]) = log10(pmat2(pmat2(:,5)==0,[3 4]));

parshat = pmat(:,1);            % this is the parameter vector
par_sel = pmat(:,2);            % this is the selection vector
Xhat    = pmat(parnum,1);       % the best estimate for the parameter (2nd column is select)

%% Set performance parameters
% These values can be changed if the performance of the likelihood
% procedure is not adequate.

chicrit = 3.8415;% = chi2inv(0.95,1); % the chi square criterion for 1 df and 95%
% This is coded in hard form so that this function can operate without the
% statistics toolbox.

switch prof_detail
    case 1
        Fstep_min = 0.001;    % min stepsize (as fraction of value that is tried)
        Fstep_max = 30;       % max stepsize (as fraction of value that is tried)
        if Xhat == 0,         % if parameter value is zero; than parameter is likely a NEC
            Fstep_abs = 1e-4; % This is just a low value that should be ok in most situations
        else
            Fstep_abs = 1e-3 * abs(Xhat); % smallest stepsize (in absolute sense)
        end
        % (this is to prevent problems when the parameter value that is tried goes
        % to zero; the stepsize is a fraction of the value tried so can also become
        % very small).
        
        % criteria for change in likelihood ratio
        Lcrit_max  = 1;   % maximum change in likelihood, above this value, stepsize is decreased
        Lcrit_min  = 0.2; % minimum change in likelihood, below this value, stepsize is increased
        Lcrit_stop = 15;  % stop when likelihood ratio reaches this value
        
    case 2
        Fstep_min = 0.005;    % min stepsize (as fraction of value that is tried)
        Fstep_max = 30;       % max stepsize (as fraction of value that is tried)
        if Xhat == 0,         % if parameter value is zero; than parameter is likely a NEC
            Fstep_abs = 1e-3; % This is just a low value that should be ok in most situations
        else
            Fstep_abs = 1e-2 * abs(Xhat); % smallest stepsize (in absolute sense)
        end
        % (this is to prevent problems when the parameter value that is tried goes
        % to zero; the stepsize is a fraction of the value tried so can also become
        % very small).
        
        % criteria for change in likelihood ratio
        Lcrit_max  = 2; % maximum change in likelihood, above this value, stepsize is decreased
        Lcrit_min  = 0.7; % minimum change in likelihood, below this value, stepsize is increased
        Lcrit_stop = 10;  % stop when likelihood ratio reaches this value
        
end

% set options for the fitting (rougher than for finding the best fit; increases speed)
oldopts  = optimset('fminsearch');
options  = optimset(oldopts,'Display','off','FunValCheck','on'); % show no iterations
options1 = optimset(options,'TolX',1e-2,'TolFun',1e-2,'MaxFunEvals',30*(length(parshat)-1)); % only do rough optimisation
options2 = optimset(options,'TolX',1e-2,'TolFun',1e-2,'MaxFunEvals',50*(length(parshat)-1)); % only do rough optimisation

%% Start with the profiling ...

par_sel(parnum) = 0; % do not fit this parameter anymore, as we make profile
pmat_orig = pmat;    % remember the original parameter values (we are messing with a copy)
pmat(:,2) = par_sel; % overwrite the selection part of pmat with the new par_sel

p_index = (par_sel==1); % find the parameters that need to be fitted
loglikmax = -1 * transfer(parshat(p_index),pmat); % use transfer to obtain max likelihood!
Xhat = parshat(parnum); % best value of the profiled parameter

parmin  = pmat2(parnum,3); % use min and max of parameter as provided in script
parmax  = pmat2(parnum,4); % taken from pmat2, so these may be on log-scale

for j = 1:2, % calculate from profile down and up from the ML estimate
    if j == 1,
        % First calculating from max. lik. estimate downwards to zero ...
        logliklowest = 0;
        proffact     = -1; % going down ...
        
        figure % initialise a figure to keep track of the progress
        h1 = gca; % remember the current axis number for plotting!
        hold on
        plot(h1,Xhat,0,'ko','MarkerFaceColor','w') % plot the max lik value as a circle
        drawnow % update the plot
        
        % if the parameter was on log-scale, mention it in the axis label
        if pmat(parnum,5) == 0
            xlabel(h1,['parameter ',names{parnum},' (log-scale)'],'FontSize',12)
        else
            xlabel(h1,['parameter ',names{parnum}],'FontSize',12)
        end
        
        ylabel(h1,'minus 2x log-likelihood ratio','FontSize',12)
        set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting
        drawnow
        
    else
        % Now starting calculations from max. lik. estimate upwards ...
        proffact = 1; % going up ...
        % remember values from the first run (downwards) and clear 'em
        XcollD = Xcoll;
        loglikratD = loglikrat;
        clear Xcoll loglikrat;
    end
    
    p_try     = pmat; % temporary parameter vector
    Xtry      = Xhat; % initialise the parameter to try on the ML estimate
    logliktmp = 0;
    i         = 1;
    
    % first values are zero
    Xcoll(1)     = Xtry; % all tried parameter values will be stored in Xcoll
    loglikrat(1) = 0;
    flag         = 0;
    
    % initialise Fstep as fraction of parameter value
    Fstep   = Fstep_min; % start from the minimum step size
    flagmin = 0; % test: flag is used to indicate that stepsize cannot or should not be decreased
    
    % this flag is put here to avoid trouble when the profile is followed
    % upwards. Each step, Fstep * Xtry will be higher than Fstep_abs, which
    % leads to a rejection of the value, a decrease in stepsize and a new
    % try. 
   
    while flag == 0, % as long as we have not reached a stopping criterion
        
        flag_try = 0;
        % remember the previous values
        Xtry_old      = Xtry;
        logliktmp_old = logliktmp;
        
        while flag_try == 0 && flag == 0; % as long as we haven't got a good new value
            
            if Xtry_old ==0, % if old tryvalue is zero, we cannot use the relative stepsize.
                Xtry    = Xtry_old + proffact * Fstep_abs; % try a new value, based on absolute stepsize
                Fstep   = Fstep_min; % and re-initialise the relative stepsize
                flagmin = 1;
            elseif Fstep * abs(Xtry_old) >= Fstep_abs, % new stepsize is larger than abs min size
                Xtry = Xtry_old + proffact * Fstep * abs(Xtry_old); % try a new value for the parameter
                % use the given relative stepsize, or the minimum absolute stepsize
                
                % now, if we are at least 1.5 times above the absolute
                % minimum and 1.5 times above the minimum relative stepsize
                % ... then it makes sense to allow for a decrease in
                % stepsize. Thus, flag is set to zero.
                if Fstep * abs(Xtry_old) > 1.5 * Fstep_abs && Fstep > 1.5 * Fstep_min,
                    flagmin = 0;
                else
                    flagmin = 1;
                end
            elseif Fstep * abs(Xtry_old) < Fstep_abs,
                Xtry    = Xtry_old + proffact * Fstep_abs; % try a new value for the parameter
                Fstep   = Fstep_abs / abs(Xtry_old);
                flagmin = 1;
            end
            
            % dont profile to negative values or very high ones!
            % make sure that values are within the specified parameter bounds
            % (bounds specified in calcstart.m)
            Xtry = max(Xtry,parmin);
            Xtry = min(Xtry,parmax);
            
            p_try(parnum,1) = Xtry ; % put new profile value into the total try vector
            p_fit = p_try(p_index,1); % these are the ones that must be estimated now
            
            % Now do fminsearch to get new best fit, with profile parameter fixed
            % Do the optimisation twice: once rough, and once a little more
            % precise. Hopefully, this will avoid local minima, but still
            % be rapid enough!
            parshattemp = fminsearch('transfer',p_fit,options1,p_try);
            [parshattemp,logliknew] = fminsearch('transfer',parshattemp,options2,p_try);
            % minimisation is done with the reduced parameter set, other parameters in par are kept
            % to the fixed value. This means that parshat will also contain the limited set only.
            
            % and make the vector whole again
            p_try(p_index,1) = parshattemp ; % replace values in total vector with values that were fitted
            logliktmp        = -2 * (-1*logliknew-loglikmax); % likelihood ratio that follows a chi square
            deltaL           = abs(logliktmp - logliktmp_old); % difference with previous likelihood
            
            % If the change in likelihood ratio is larger than we allow, or
            % if we are at the border of the allowable parameter range, we
            % want to decrease the stepsize. If the smalles stepsize is
            % already used, we can stop the analysis.
            skippit = 0;
            
            if deltaL > Lcrit_max,
                if flagmin == 0, % & logliktmp < 5, 
                    % Only reject try and decrease stepsize when the flag says that it is possible!
                    % Test: and only when we are in an interesting range of likelihood values!
                    Fstep   = max(Fstep / 2 , Fstep_min); % try again with half the stepsize, unless less than min step
                    skippit = 1; % no storage for this one!
                elseif any(Xtry == [parmin parmax]),
                    flag = 1; % if already using smallest stepsize, stop!
                end
            elseif any(Xtry == [parmin parmax]),% also stop when max or min is reached and deltaL < Lcrit_max!
                flag = 1; % stop!
            end
            
            % If the change is small enough, or if we want to stop the analysis anyway, store the value. 
            if skippit == 0,
                flag_try = 1;
                i = i+1; 
                loglikrat(i) = logliktmp; % this is 2x log-likelihood ratio
                Xcoll(i)     = Xtry; % also remember the value of c0
                plot(h1,Xcoll(i),loglikrat(i),'k.') % plot em on the fly!
                drawnow
                
                if logliktmp < logliklowest, % if we found a lower likelihood, remember it!
                    logliklowest = logliktmp;
                    p_disp       = p_try;
                    loglik_disp  = logliknew;
                    
                    % save it to the file profiles_newopt.out
                    ti = clock; % take current time
                    fprintf(fid,'%s (%02.0f:%02.0f) \n',date,ti(4),ti(5));
                    fprintf(fid,'The profiles found a better optimum: %1g (best was %1g). \n',loglik_disp,-1*loglikmax);
                    fprintf(fid,'Parameter values for new optimum \n');
                    fprintf(fid,'=================================================================================\n');
                    nfields = length(names);
                    for ii = 1:nfields,
                        if pmat(ii,5) == 0
                            fprintf(fid,'%-6s %10.4g (fit: %1.0f, initial: %1.4g) \n',names{ii},10^p_disp(ii),pmat(ii,2),10^pmat_orig(ii,1));
                        else
                            fprintf(fid,'%-6s %10.4g (fit: %1.0f, initial: %1.4g) \n',names{ii},p_disp(ii),pmat(ii,2),pmat_orig(ii,1));
                        end
                    end
                    fprintf(fid,'=================================================================================\n');
                    fprintf(fid,'  \n');
                end
                
                if logliktmp > Lcrit_stop,% stop if probability is high enough
                    flag = 1;
                elseif deltaL < Lcrit_min, % but change in likelihood is very small
                    Fstep = min(Fstep * 2 , Fstep_max); % so double the stepsize, unless greater than max step
                end
            end
        end
    end
end
fclose(fid); % close the file profiles_newopt.out for writing

%% Calculating the 95% interval and print it on-screen

diary ('results.out') % collect screen output in the diary "results.out"

if logliklowest < 0, % if the profiler found a better optimum, display it!
    fprintf('\n  The profiles found a better optimum: %1g (best was %1g). \n',loglik_disp,-1*loglikmax);
    fprintf('Parameter values for new optimum \n');
    fprintf('=================================================================================\n');
    nfields = length(names);
    for i = 1:nfields,
        if pmat(i,5) == 0
            fprintf('%-6s %10.4g (fit: %1.0f, initial: %1.4g) \n',names{i},10^p_disp(i),pmat(i,2),10^pmat_orig(i,1))
        else
            fprintf('%-6s %10.4g (fit: %1.0f, initial: %1.4g) \n',names{i},p_disp(i),pmat(i,2),pmat_orig(i,1))
        end
    end
    fprintf('=================================================================================\n');
    fprintf('  \n');
end

XcollU = Xcoll;
loglikratU = loglikrat;

disp('  ')
disp('Confidence interval from the profile')
disp('=================================================================================')


teller  = 0;
if loglikratD(end) < chicrit, % if the end of the profile is below the criterion
    teller = teller + 1; % count it as a boundary
    confD(teller) = min(XcollD); % take boundary as lower bound of conf range!
    upsw = 0; % tell the model that we are below the criterion
    disp('Warning: taking the lowest parameter value as lower confidence limit')
else 
    upsw = 1; % tell the model that we are above the criterion 
end
for i = length(loglikratD):-1:1,
    if loglikratD(i) < chicrit && upsw == 1,
        upsw = 0; % set switch to zero, we are now under the chi2 criterion
        teller = teller + 1; % found a zero crossing
        confD(teller) = interp1(loglikratD(i:i+1),XcollD(i:i+1),chicrit,'linear'); % store this value for X
    elseif loglikratD(i) > chicrit && upsw == 0, % we were below the criterion but now cross up again
        upsw = 1;
        teller = teller + 1;
        confD(teller) = interp1(loglikratD(i:i+1),XcollD(i:i+1),chicrit,'linear'); % store this value for X
    end
end

teller  = 0;
if loglikratU(end) < chicrit, % if the end of the profile is below the criterion
    teller = teller + 1; % count it as a boundary
    confU(teller) = max(XcollU); % take boundary as upper bound of conf range!
    upsw = 0; % tell the model that we are below the criterion
    disp('Warning: taking the highest parameter value as upper confidence limit')
else 
    upsw = 1; % tell the model that we are above the criterion 
end
for i = length(loglikratU):-1:1,
    if loglikratU(i) < chicrit && upsw == 1,
        upsw = 0; % set switch to zero, we are now under the chi2 criterion
        teller = teller + 1; % found a zero crossing
        confU(teller) = interp1(loglikratU(i:i+1),XcollU(i:i+1),chicrit,'linear'); % store this value for X
    elseif loglikratU(i) > chicrit && upsw == 0, % we were below the criterion but now cross up again
        upsw = 1;
        teller = teller + 1;
        confU(teller) = interp1(loglikratU(i:i+1),XcollU(i:i+1),chicrit,'linear'); % store this value for X
    end
end

% if the parameter was on log-scale, now back-convert the CI
if pmat(parnum,5) == 0
    confD = 10.^(confD);
    confU = 10.^(confU);
end

if length(confD) == 1 && length(confU) == 1,
    % no problems, it is a single interval
    fprintf('%-10s interval: %10.4g - %1.4g \n',names{parnum},confD,confU)
else
    disp('The confidence interval is a broken set (check likelihood profile to check these figures)')
    fprintf('%-10s \n',names{parnum})
    if length(confD)>1,
        for i = 1:(length(confD)-1)/2,
            fprintf('   interval: %10.4g - %1.4g \n',confD(i*2-1),confD(i*2))
        end
    end
    fprintf('   interval: %10.4g - %1.4g \n',confD(end),confU(end))
    if length(confU)>1,
        for i = 1:(length(confU)-1)/2,
            fprintf('   interval: %10.4g - %1.4g \n',confU(i*2),confU(i*2-1))
        end
    end
end

% And make a nice plot of the profile 
plot(h1,XcollD,loglikratD,'k-')
plot(h1,XcollU,loglikratU,'k-')

% Plot the chi-square criterion for 95% and 1 degree of freedom
plot(h1,[min(XcollD) max(XcollU)],[chicrit chicrit],'k:')
drawnow

% set the y-axis to allow to read the graph when lik-ratio becomes very large
plotmin = min([loglikratD loglikratU]);
plotmax = max(10,min(30,max([loglikratD loglikratU]))); % force plotmax between 10 and 30
ylim(h1,[plotmin plotmax])

disp(' ')
disp(['Time required: ' secs2hms(toc)])
diary off  % close results.out