function [minloglik,GoodFit] = transfer(pfit,pmat)

% This is a transfer function which calculates the minus log-likelihood
% function for a certain set of parameters.
%
% This transfer function allows certain parameters to be fixed, while
% others are fitted. The free parameters are in <pfit>, all parameters in
% <p_mat>, which contains in the second column a 1 where the parameter is to
% be fitted. <X0mat> is the matrix with initial states.
%
% This function is used by start_calc.m when fminsearch is called, and by
% calc_proflik.m. 
%
% Author : Tjalling Jager 
% Date: July 2015
% Web support: http://www.debtox.info/byom.html

global DATA W X0mat ttot ctot pri glo zvd n_D n_X % <X0mat> is the matrix with initial states.
global n_X2 DATAx Wx % globals for the optional extra data sets

sameres = 0; % use one overall nuisance residual sd for all datasets for one state (this needs further testing!)
Fsdmin  = 0; % option to force a minumum value of the SSQ to avoid focus on perfect fit (try 0.001)

p = pmat(:,1);           % copy the whole parameter vector to p
p(pmat(:,2)==1) = pfit ; % replace values in p with the values to be fit (pfit)

% put parameters that need to be fitted on log scale back on normal scale
p(pmat(:,5)==0) = 10.^(p(pmat(:,5)==0));
% pmat needs to be on normal scale for call_deri

% if any parameter is outside the bounds, return +inf as criterion
if any(p<pmat(:,3) | p>pmat(:,4)),
    minloglik = +inf; % than make the minloglik infinite (very bad fit!)
    return; % and return to the function calling this one
end

ndata = n_X * n_D; % total number of data sets
% initialise the matrices that will collect the model output
Xcoll = cell(n_X,1); % create empty cell array
for i = 1:n_X,     % loop over the data sets
    Xcoll{i} = zeros(length(ttot),length(ctot)); % and fill an Xcoll with zeros
end
if n_X2 > 0 % if there are additional univariate data ...
    for j = 1:n_X2, % now loop over the additional data sets
        Xcoll2x{j} = zeros(length(ttot),length(ctot)); % and fill an Xcoll2x with zeros
        Xcoll2y{j} = zeros(length(ttot),length(ctot)); % and fill an Xcoll2y with zeros
    end
end

par = packunpack(2,0,p); % transform parameter matrix into a structure

% =======================================================================
%               GET MODEL RESULTS
% =======================================================================

for i = 1:length(ctot) % run through all of our concentrations
    % initial states to start the solver with, from X0mat
    X0 = X0mat(2:end,X0mat(1,:) == ctot(i)); % read initial states from X0mat using "concentration" vector
    
    if n_X2 > 0 % if there are additional univariate data, we need an extra output ...
        [Xout TE Xout2] = call_deri(ttot,par,[ctot(i);X0]); % use call_deri.m to provide the output per concentration
    else % this is needed to ensure backwards compatability
        [Xout TE] = call_deri(ttot,par,[ctot(i);X0]); % use call_deri.m to provide the output per concentration
    end
   
    if size(Xout,1) == length(ttot)
        for j = 1:n_X, % now loop over the state variables
            Xcoll{j}(:,i) = Xout(:,j); % and collect the results in the right part of Xcoll
        end
    else % call_deri did not return all required time points, so something went wrong in calculating the derivatives
        minloglik = +inf; % than make the minloglik infinite (very bad fit!)
        return; % and return to the function calling this one
    end
    for j = 1:n_X2, % now loop over the additional data sets
        Xcoll2x{j}(:,i) = Xout2{j}(:,1); % collect the new x-values
        Xcoll2y{j}(:,i) = Xout2{j}(:,2); % collect the new y-values
    end
end

% =======================================================================
%               CALCULATE LIKELIHOOD
% =======================================================================

loglik = zeros(ndata,1); % initialise a vector for the log-likelihoods with zeros
find_data = reshape(1:ndata,n_D,n_X); % helper to find which state a data set is
if n_D > 1 && sameres == 1
    i_rem = 1; % perhaps also initialise rem_ssq?
end

for i = 1:ndata, % loop over the data sets
    
    if size(DATA{i},1) == 1 % if the DATA is a row vector (only scenarios, or just a zero), ignore it
        loglik(i) = 0;
    else
        
        D   = DATA{i}(2:end,2:end); % extract the correct data matrix
        tD  = DATA{i}(2:end,1);     % time vector in this data set
        cD  = DATA{i}(1,2:end);     % concentration vector in this data set
        lam = DATA{i}(1,1);         % -1 for survival data, 0-1 for continuous data (data transformation when lam<1)
        w   = W{i};                 % weight factors are by default set to 1 in start_calc
        
        [dummy,locC]=ismember(cD,ctot); % locC is location in ctot for common values with cD
        if any(locC==0) % it is possible that the model scenarios do not cover all scenarios in the data
            D(:,locC==0)  = []; % remove columns of data that we do not want to fit (they are not in ctot)
            w(:,locC==0)  = []; % remove columns of weights that we do not want to fit (they are not in ctot)
            locC(locC==0) = []; % also remove the zeros from the location vector
        end
        [dummy,locT]=ismember(tD,ttot); % locT is location in ttot for common values with tD
        if any(locT==0) % if we set glo.Tinit, it is possible that the model times do not cover all scenarios in the data
            D(locT==0,:)  = []; % remove columns of data that we do not want to fit (they are not in ctot)
            w(locT==0,:)  = []; % remove columns of weights that we do not want to fit (they are not in ctot)
            locT(locT==0) = []; % also remove the zeros from the location vector
        end
        
        if ~isempty(D) % D will be made empty when the scenarios do not match the data at all
            
            if n_D == 1
                M = Xcoll{i}(locT,locC);    % model values at the concentration and timepoint of the data
                % this also works when there are replicated data; the model values
                % are simply copied to the correct columns!!
            else
                [i_D,i_st] = find(find_data==i); % find the state that data set i belongs to
                M = Xcoll{i_st}(locT,locC);    % model values at the concentration and timepoint of the data
            end
            
            if lam == -1, % than we have survival data
                
                if ~any(isnan(D(:))) % does the dataset NOT contain nans?
                    prob = M - [M(2:length(tD),:); zeros(1,length(locC))]; % multinomial death probabilities for each interval
                    Z    = D-[D(2:length(tD),:);zeros(1,length(locC))]; % recalculate survivors to deaths ...
                    Z    = Z(:);    % and make a long vector out of it
                    prob = prob(:); % also one vector for the probabilities
                    w2   = w(:); % weights represent animals removed or missing
                    M    = M(:); % turn it into a column vector
                    
                else
                    % slightly different approach to deal with NaNs in survival data
                    % the trick is to turn the survivor matrix into a vector,
                    % remove the NaNs entirely, than do the calculation of
                    % death probabilities and number of deaths
                    M(isnan(D))=NaN; % replace survival probabilities with NaN, where there are NaNs in the data
                    M2 = [M ; zeros(1,length(locC))]; % add a row with zeros at the bottom (at t=inf, survival is zero)
                    indnan = cumsum(sum(isfinite(M2),1)); % linear index to last datapoint for each conc, excl NaNs
                    indnan(end) = []; % remove last one (does not overlap with next conc)
                    M2 = M2(:); % turn it into a column vector
                    M2(isnan(M2)) = []; % remove NaNs from vector
                    prob = M2(1:end-1)-M2(2:end); % deaths from survivors
                    prob(indnan) = []; % remove extra values that arise from overlap with results for next concentration
                    
                    % Do the same for the dataset
                    D2 = [D ; zeros(1,length(locC))]; % add a row with zeros at the bottom (at t=inf, survival is zero)
                    D2 = D2(:); % turn it into a column vector
                    D2(isnan(D2)) = []; % remove NaNs from vector
                    Z = D2(1:end-1)-D2(2:end); % deaths from survivors
                    Z(indnan) = []; % remove extra values that arise from overlap with results for next concentration
                    
                    % Also remove the NaN positions in the missing/removed matrix
                    w2 = w;
                    w2(isnan(D))=NaN; % replace removed organisms with NaN, where Z is NaN
                    w2 = w2(:); % turn it into a column vector
                    w2(isnan(w2)) = []; % remove the NaNs
                    M = M(:); % turn it into a column vector
                    M(isnan(M)) = []; % remove the NaNs
                end
                
                Z = Z - w2; % correct for the number of animals removed or missing
                if any(Z<0)
                    error('Negative deaths discoverd. Please check the weights matrix in the data set.')
                end
                prob = max(prob,1e-50); % Otherwise we get problems taking the logarithm
                M = max(M,1e-50); % Otherwise we get problems taking the logarithm
                
                loglik(i) = Z'*log(prob) + w2'*log(M);  % and calculate the log likelihood
                % note that the removed or missing animals contribute too by
                % the information that they WERE alive up till some time point
                 
                % Goodness of fit measure according to Bedaux & Kooijman (1994)
                % This needs further testing ...
                Dtmp = ones(length(tD),1)*D(1,:);
                Dtmp = Dtmp(:); % make a row vector out of it
                Dtmp(isnan(D(:))) = []; % remove places where there are NaNs from vector (if any)
                Dtmp = Dtmp - w2; % simply also extract the missing/removed animals
                
                prob_sup = Z./Dtmp; % maximum probability
                prob_sup = max(prob_sup,1e-50); % Otherwise we get problems taking the logarithm
                loglik_sup = Z'*log(prob_sup);  % and calculate the log likelihood
                loglikrat = -2 * (loglik(i)-loglik_sup);
                GoodFit(i) = 1-chi2cdf(loglikrat,length(prob_sup)-length(cD)-length(pfit));
                % GoodFit(i) = 1-chi2cdf(loglikrat,length(prob_sup)-length(cD));
                % GoodFit(i) = 1-chi2cdf(loglikrat,length(prob_sup)-length(cD)-4);
                % for the DF, I subtract length(cD) as the sum of all
                % probabilities has to be 1. So, the probability to fall
                % into the last interval does not need to be estimated.
                % I also subtract the total number of fitted parameters,
                % which might NOT be such a good idea when fitting multiple
                % data sets simultaneously ... perhaps always subtract 4?
                % Might be better to have a goodness-of-fit measure that is
                % independent of the number of parameters.
                            
%                 % TEST: goodness of fit using the chi2 test
%                 % I think the previous one is better (this applies SSQ, so likely assumes normality)
%                 D_obs = Z;
%                 D_pred = prob.*Dtmp;
%                 chicrit = sum(((D_obs-D_pred).^2)./D_pred);
%                 GoodFit = 1-chi2cdf(chicrit,length(prob_sup)-length(cD)-length(pfit))
%                 % for the DF, I subtract length(cD) as the sum of all
%                 % probabilities has to be 1. So, the probability to fall
%                 % into the last interval does not need to be estimated.
                
            else % than we have continuous response data
                
                D = D(:); % put data in a single vector
                M = max(0,M(:)); % and the same for the model results
                w = w(:); % and the weight factors
                ind_fin = (isfinite(D)==1); % find where data is a nice finite number (so not NaN)
                
                if lam == 0, % than do log-transformation before taking residuals
                    res = log(D(ind_fin)+1e-6) - log(M(ind_fin)+1e-6) ; % residuals from transformed data
                    mn = mean(log(D(ind_fin)+1e-6)); % mean value of data after transformation
                    res_tot = log(D(ind_fin)+1e-6) - mn ; % residuals from mean of data
                else % than do a power transformation (none if lam=1) before taking residuals
                    res = (D(ind_fin)).^lam - (M(ind_fin)).^lam ; % residuals from transformed data
                    mn = mean(D(ind_fin).^lam); % mean value of data after transformation
                    res_tot = (D(ind_fin)).^lam - mn ; % residuals from mean of data
                end
                wssq  = res' * diag(w(ind_fin)) * res ; % weigthed sum of squares
                wssq2 = res' * (diag(w(ind_fin))).^2 * res ; % weigthed sum of squares (weighted with n squared)
                n = sum(ind_fin);    % number of data points
                N = sum(w(ind_fin)); % total nr of individuals used
                wssq_tot = res_tot' * diag(w(ind_fin)) * res_tot ; % weigthed sum of squares
                GoodFit(i) = 1 - wssq/wssq_tot; % R-square, NOT adjusted for no. of pars.
                % GoodFit(i) = 1 - (wssq/(n-length(pfit)-1))/(wssq_tot/(n-1); % R-square, adjusted for no. of pars. 
                 
                % From Wikipedia: Adjusted R2 does not have the same
                % interpretation as R2while R2 is a measure of fit,
                % adjusted R2 is instead a comparative measure of
                % suitability of alternative nested sets of explanators.
                
                if ~isfield(glo,'var')
                    % set a minimum for the wsssq, so that sd of residuals is >
                    % a factor Fsdmin (top of function) of the average of the data
                    % this should avoid focus on perfect fits
                    wssq2 = max(wssq2,N*(Fsdmin * mn)^2);
                    wssq  = max(wssq,n*(Fsdmin * mn)^2);
                    % Note: watch out when fitting means with variable number
                    % of individuals per mean (not exactly correct, so
                    % discontinuity may occur using max).
                    
                    if n_D == 1 || sameres == 0 % for one data set per state, simply calculate log likelihod
                        % calculate log-likelihood for replicated data with sd as nuisance parameter
                        loglik(i) = -(n/2) * log(wssq2) - N*wssq/(2*wssq2);
                    else % if there are more data sets that we need to combine ...
                        rem_ssq(i_rem,:) = [i_st i wssq wssq2 n N];
                        i_rem = i_rem + 1;
                    end
                else
                    % calculate log-likelihood for replicated data with sd provided in global
                    % the variance must be for individuals, and after transformation!
                    % If variance of mean is known, enter variance times number
                    % of replicates. This makes sure that differences between
                    % number of replicates can be dealt with properly.
                    loglik(i) = -1/(2*glo.var(i)) * wssq;
                end
            end
        end
    end
end

if n_D > 1 && sameres == 1 % in case we want to have a single residual sd per state ...
    rem_un = unique(rem_ssq(:,1));
    for i = 1:length(rem_un) % go through unique states in the remembered matrix with ssq's and n's
        ind_i = find(rem_ssq(:,1)==rem_un(i));
        rem_i = sum(rem_ssq(ind_i,3:end),1);
        % calculate a log-likelihood from the summed results for this state
        % and place it in loglik at first position for this state
        loglik(rem_ssq(ind_i(1),2)) = -(rem_i(3)/2) * log(rem_i(2)) - rem_i(4)*rem_i(1)/(2*rem_i(2));
    end
end

if isfield(glo,'wts') % if weights for the data sets are specified ...
    loglik = loglik .* glo.wts(:); % use weights for each data type
    % Note: when using a common residual error for multple data sets within
    % a state, only the first weight for that state will be used!
end

logpriprob = 0; % initiate prior probability
if ~isempty(pri)
    logpriprob = calc_prior(par); % call function to calculate prior prob.
end

logzvdprob = 0; % initiate zero-variate data probability
if ~isempty(zvd)
    logzvdprob = calc_zerovar; % call function to calculate zero-variate data prob.
end

% Next, calculate the log-likelihood for any additional data sets
loglik2 = zeros(n_X2,1); % initialise a vector for the log-likelihoods with zeros
for i = 1:n_X2, % loop over the additional data sets (not done when n_X2 = 0)
    
    if size(DATAx{i},1) == 1 % if the DATA is a row vector (only scenarios, or just a zero), ignore it
        loglik2(i) = 0;
    else
        
        D   = DATAx{i}(2:end,2:end); % extract the correct data matrix
        xD  = DATAx{i}(2:end,1);     % x vector in this data set
        cD  = DATAx{i}(1,2:end);     % concentration vector in this data set
        lam = DATAx{i}(1,1);         % -1 for survival data, 0-1 for continuous data (data transformation when lam<1)
        w   = Wx{i};                 % weight factors are by default set to 1 in start_calc

        [dummy,locC]=ismember(cD,ctot); % locC is location in ctot for common values with cD
        if any(locC==0) % it is possible that the model scenarios do not cover all scenarios in the data
            D(:,locC==0)  = []; % remove columns of data that we do not want to fit (they are not in ctot)
            w(:,locC==0)  = []; % remove columns of weights that we do not want to fit (they are not in ctot)
            locC(locC==0) = []; % also remove the zeros from the location vector
        end
        
        if ~isempty(D) % D will be made empty when the scenarios do not match the data at all
            
            Mx = Xcoll2x{i}(:,locC);    % model x-values at the concentration of the data
            My = Xcoll2y{i}(:,locC);    % model y-values at the concentration of the data
            % this also works when there are replicated data; the model values
            % are simply copied to the correct columns!!
            M = [];
            for j = 1:size(Mx,2) % run through scenarios
                M(:,j) = interp1(Mx(:,j),My(:,j),xD,'linear','extrap'); % linear inter- and extrapolation
                % to get model values at the exact x-values of the data set
            end
            
            D = D(:); % put data in a single vector
            M = max(0,M(:)); % and the same for the model results
            w = w(:); % and the weight factors
            ind_fin = (isfinite(D)==1); % find where data is a nice finite number (so not NaN)
            
            if lam == 0, % than do log-transformation before taking residuals
                res = log(D(ind_fin)+1e-6) - log(M(ind_fin)+1e-6) ; % residuals from transformed data
                mn = mean(log(D(ind_fin)+1e-6)); % mean value of data after transformation
            else % than do a power transformation (none if lam=1) before taking residuals
                res = (D(ind_fin)).^lam - (M(ind_fin)).^lam ; % residuals from transformed data
                mn = mean(D(ind_fin).^lam); % mean value of data after transformation
            end
            wssq  = res' * diag(w(ind_fin)) * res ; % weigthed sum of squares
            wssq2 = res' * (diag(w(ind_fin))).^2 * res ; % weigthed sum of squares (weighted with n squared)
            n = sum(ind_fin);    % number of data points
            N = sum(w(ind_fin)); % total nr of individuals used
        
            % set a minimum for the wsssq, so that sd of residuals is >
            % a factor Fsdmin (top of function) of the average of the data
            % this should avoid focus on perfect fits
            wssq2 = max(wssq2,N*(Fsdmin * mn)^2);
            wssq  = max(wssq,n*(Fsdmin * mn)^2);
            % Note: watch out when fitting means with variable number
            % of individuals per mean (not exactly correct, so
            % discontinuity may occur using max).
            
            % calculate log-likelihood for replicated data with sd as nuisance parameter
            loglik2(i) = -(n/2) * log(wssq2) - N*wssq/(2*wssq2);
           
        end
    end
end

if isfield(glo,'wts2') % if weights for the extra data sets are specified ...
    loglik2 = loglik2 .* glo.wts2(:); % use weights for each data type
end

% Calculate the overall log likelihood
minloglik = -1 * (sum(loglik)+logpriprob+logzvdprob+sum(loglik2)) ; % makes it negative. This is needed as fminsearch is looking for a MINIMUM,
% and we need to find the MAXIMUM of the likelihood function.

if isinf(minloglik) % the minloglik should not be calculated as infinite ...
    error('The min-log-likelihood is calculated to be infinite ... check data set and derivatives or debug from transfer.m')
end

if isnan(minloglik) || ~isreal(minloglik) % when the loglik calculation returns NaN or complex numbers ...
    minloglik = +inf; % give it a really bad likelihood value so the optimisation ignores it
end