% Build-Yer-Own-Model
% 
% General framework for simulating model systems in terms of ordinary
% differential equations (ODEs). This script requires call_deri.m and
% derivatives.m to be specified. Additionally, for fitting, the files
% start_calc.m, packunpack.m, transfer.m and sec2hms.m are used. The
% function calc_proflik.m is optional and can be called to calculate
% profile likelihoods.
%
% Fitting relies on the multinomial likelihood for survival and independent
% normal distributions (if needed after transformation) for continuous
% data. Results are shown on screen but also saved to a log file
% (results.out).
%
% Author: Tjalling Jager (www.debtox.info)
% Date:   Dec 2014

% modified by starrlight augustine march 17 2015
% guts workshop Mallorca

% ========================================================================
%                         THIS EXAMPLE
% ========================================================================
% byom_guts2: fitting survival data with the GUTS special cases based on 
% the (scaled) internal concentration: SD, IT and mixed (or GUTS proper).
% The use of a damage stage is not implemented in this example, but is 
% included in the in this package in the subfolder 'damage'.
%
% For the NEC distribution, the log-logistic or the log-normal can be used.
% As long as the exposure concentration is constant, the analytical 
% solution in simplefun.m can be used (in call_deri.m, set useode=0), which
% is much faster than the ODE in derivatives (which is adapted in other 
% examples to use time-varying exposure).
% ========================================================================

clear all           % clear the memory
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
global pri zvd      % global structures for optional priors and zero-variate data
diary off           % turn of the diary function (if it is accidentaly on)
set(0,'DefaultFigureWindowStyle','docked') ; 
% use this option to collect all figure into one window with tab controls

% Set path to the BYOM/engine directory. The function pathdefine
% only works when the mydata file is in a folder below the BYOM folder.
pathdefine

% ========================================================================
%                       ENTER DATA SET
% ========================================================================

% Data are entered in matrix form, time in rows, scenarios (exposure
% concentrations) in columns. First column are the exposure times, first
% row are the concentrations or scenario numbers.

% Number top left of matrix indicates how to calculate the likelihood:
% -1 ) multinomial likelihood (for survival data)
% 0  ) log-transform the data, then normal likelihood
% 0.5) square-root transform the data, then normal likelihood
% 1  ) no transformation of the data, then normal likelihood

% For each state variable there should be a data set in the correct
% position. So, DATA{1} should be the data for state variable 1, etc. 
% The curly braces indicate that this is a 'cell array'. If there are
% no data for a state, use DATA{i}=0. Use NaN for missing data.

% observed number of survivors, time in days, conc in nmol/L

N = [-1	0	756.658595642	1513.317191283	3026.634382567;
    1	1	1	1	1
    7	1	0.97	0.97	0.98
    14	0.97	0.83	0.71	0.68];
N(2:end,2:end) = N(2:end,2:end).*12;

DATA{1} = N;

% internal concentrations
DATA{2} = 0;   

% Initial states, scenarios in columns, states in rows
X0mat(1,:) = [0 756.658595642	1513.317191283	3026.634382567];   % nmol/L scenarios (concentrations)
X0mat(2,:) = 1; % initial survival probability
X0mat(3,:) = 0; % initial internal concentration

% ========================================================================
%                   INITIAL VALUES FOR MODEL PARAMETERS 
% ========================================================================
  
% model parameters are part of a `structure` for easy reference
% syntax: par.name = [startvalue fit(0/1) minval maxval];

% if parameter K is not specified, package will use the scaled TK model
% if parameter Ci0 is not specified, package will use value in X0mat as starting value
 
par.ke    = [0.001  1 0.001 10]; % elimination rate constant (d-1)
par.z     = [1.75   1 0 1e6];    % NEC for survival (ug/L)
par.hb    = [0.0018 1 0 1e6];    % background hazard rate (1/d)
par.kk    = [0.0018 1 0 1000];   % killing rate (L/ug/d) (SD and mixed)
par.Fs    = [2      0 1 100];    % fraction spread of NEC distribution (-) (IT and mixed)

% global parameter
glo.sel   = 1; % select death mechanism: 1) SD 2) IT 3) mixed
glo.dis   = 1; % NEC distribution for IT: 1) log-logistic 2) log-normal
% globals used for GUTS purposes
glo.locDM = 2; % location of the dose metric in the state variable list
glo.locS  = 1; % location of survival probability in the state variable list

switch glo.sel % make sure that right parameters are fitted
    case 1 % SD
        par.Fs(2) = 0; % never fit the NEC spread!
    case 2 % IT
        par.kk(2) = 0; % never fit the killing rate!
    case 3 % mixed
        % do nothing: fit all parameters
end

% specify what to calculate and what to plot
t = linspace(0,15,100); % time vector for the model curves in days
fit   = 1; % set to 1 to fit the parameters, 0 to simulate only
sho   = 1; % set to 1 to show all scenarios, 0 to only plot model for scenarios with data
repls = 1; % set to 1 to plot replicates, 0 to plot mean responses 

% specify the y-axis labels for each state variable
ylab{1} = 'survival probability';
ylab{2} = 'scaled internal concentration (nmol/L)';
% specify the x-axis label (same for all states)
xlab    = 'time (days)';
leglab1 = 'Conc. '; % legend label before the 'scenario' number
leglab2 = ' nmol/L'; % legend label after the 'scenario' number

% ========================================================================
%                CALCULATE AND PLOT
% ========================================================================

mfilename_base = mfilename; % remember the filename for THIS file for the plots
calc_and_plot; % call the script which calculates and plots

% ========================================================================
%                STANDARD ERRORS AND CONFIDENCE INTERVALS
% ========================================================================
% By profiling you make robust confidence intervals for one or more of your
% parameters. Use the name of the parameter as it occurs in your parameter
% structure "par" above. You do not need to run the entire script before
% you can make a profile. However, make sure that you have run at least one
% optimisation before you make profiles (start_calc sets globals that are 
% needed for this calculation too).
% Note 1: profiling for complex models is a slow process, so grab a coffee!
% Note 2: if the profile finds a better solution, it immediately saves it
% to the lof file profiles_newopt.out. You can break of the analysis by
% pressing ctrl-c and use the values from the log file to restart.
 
% calc_proflik(par_out,'ke');  % uncomment to calculate a profile
% calc_proflik(par_out,'z');   % uncomment to calculate a profile
% calc_proflik(par_out,'kk');  % uncomment to calculate a profile
% calc_proflik(par_out,'hb');  % uncomment to calculate a profile
