function [Xout TE] = call_deri(t,par,X0v)

% This function calls the ODE solver to solve the system of differential
% equations specified in derivatives.m. As input, it gets:
% <t>     the time vector
% <par>   the parameter structure
% <X0v>   a vector with initial states and the concentration (scenario number)
% 
% The output <Xout> provides a matrix with time in rows, and states in columns.
% This function calls derivatives.m. The optional output <TE> is the time at
% which an event takes place (specified using the events function)
%
% The events function is set up to catch discontinuities. It should be
% specified according to the problem you are simulating! If you want to use
% parameter that are (or influence) initial states, they have to be
% included in this function.
%
% Author: Tjalling Jager (www.debtox.info)
% Date  : Dec 2014

global glo   % allow for global parameters in structure glo
global zvd   % global structure for zero-variate data

% =========================================================================
%                   USER-MODIFIABLE SECTION
% =========================================================================

useode   = 0; % set to 1 solves the set of ODEs in derivatives, zero expects 
              % an explicit function in simplefun
eventson = 1; % set to 1 uses the events function: also modify the 
              % sub-function at the bottom of this function!

% Unpack the vector X0v, which is X0mat for one scenario
X0 = X0v(2:end); % these are the intitial states for a scenario
% if needed, extract parameters from par that influence initial states in X0
if isfield(par,'Ci0') % if there is an initial concentration specified ...
    Ci0   = par.Ci0(1); % parameter for the internal concentration
    X0(2) = Ci0; % put this parameter in the correct location of the initial vector
end

% % if needed, calculate model values for zero-variate data from parameter set
% if ~isempty(zvd)
%     zvd.ku(3) = par.Piw(1) * par.ke(1); % add model prediction as third value
% end

% =========================================================================
%                   GENERALLY NO NEED TO MESS WITH FILES HERE
% =========================================================================

c  = X0v(1);     % the concentration (or scenario number)
t  = t(:);       % force t to be a row vector (needed when useode=0)

% specify options for the ODE solver
options = odeset; % start with default options
if eventson == 1
    options = odeset(options, 'Events',@eventsfun); % add an events function
end
options = odeset(options, 'RelTol',1e-4,'AbsTol',1e-7); % specify tightened tolerances
options = odeset(options,'InitialStep',max(t)/1000,'MaxStep',max(t)/100); % specify smaller stepsize

TE = 0; % dummy for time of events

if useode == 1 % use the ODE solver to calculate the solution
    % call the ODE solver (use ode15s for stiff problems, escpecilly for pulses)
    if isempty(options.Events) % if no events function is specified ...
        % [tn, Xout] = ode45(@derivatives, t, X0, options, par, c);
        [tn, Xout] = ode15s(@derivatives, t, X0, options, par, c);
    else % with an events functions ...
        % [tn, Xout, TE, YE, IE] = ode45(@derivatives, t, X0, options, par, c);
        [tn, Xout, TE, YE, IE] = ode15s(@derivatives, t, X0, options, par, c);
        % additional output arguments for events:
        % TE catches the time of an event, YE the states at the event, and IE the number of the event
    end
else % alternatively, use an explicit function provided in simplefun!
    Xout = simplefun(t,X0,par,c);
end

if isempty(TE) || all(TE == 0) % if there is no event caught
    TE = +inf; % return infinity
end

% =========================================================================
%                   OUTPUT MAPPING
% =========================================================================
% Xout contains a row for each state variable. It can be mapped to the data
% If you need to transform the model values to match the data, do it here.
% Here, this part is heavily modified to allow for IT and GUTS-proper
% calculations (in which both SD and IT are combined).

if glo.sel ~= 1 % for IT and the mixed model ...
    z     = par.z(1); % no-effect concentration
    Fs    = max(1+1e-6,par.Fs(1)); % fraction spread of the NEC distribution
    sd_z  = log10(Fs)/1.96;  % sd on 10log scale, calculated from Fs
    beta  = log(39)/log(Fs); % shape parameter for logistic from Fs
end

% This fast way for GUTS proper uses the ODEs to calculate the dose metric
% only (which the same for all individuals). Survival for different NECs is
% calculated here. Note that for the mixed model, a long time vector will
% be generated in start_calc, which is needed to get an accurate
% integration here.
if glo.sel == 3 % for mixed model, rapid way ...
    n = 200; % number of slices from the NEC distribution
    kk = par.kk(1); % killing rate
    if glo.dis == 1 % log-logistic distribution
        Fs2 = 999^(1/beta); % fraction spread for 99.9% of the distribution
        z_range = linspace(z/(1.5*Fs2),z*Fs2,n); % range of NECs to cover 99.9%
        prob_range = ((beta/z)*(z_range/z).^(beta-1)) ./ ((1+(z_range/z).^beta).^2); % pdf for the log-logistic (Wikipedia)
    elseif glo.dis == 2 % lognormal distribution
        logz_range = linspace(log10(z)-3*sd_z,log10(z)+3*sd_z,n); % regular range of log NECs
        z_range = 10.^logz_range; % back to normal scale
        prob_range = normpdf(logz_range,log10(z),sd_z); % with their prob densities
    end
    prob_range = prob_range / sum(prob_range); % normalise the densities to exactly one   
    S = zeros(length(t),1); % initialise the survival probability over time with zeros
    DM = Xout(:,glo.locDM); % take the correct state variable as dose metric
    for i = 1:n % run through the different NECs
        haz = kk * max(0,DM-z_range(i)); % calculate hazard for each NEC
        cumhaz = cumtrapz(t,haz); % integrate the hazard rate numerically
        surv = min(1,exp(-1*cumhaz)); % calculate survival probability
        S = S + surv * prob_range(i); % add to Stemp, weighted for this NECs prob density
    end
    S = S .* Xout(:,glo.locS); % make sure to add background hazard
    Xout(:,glo.locS) = S; % replace correct state by newly calculated survival prob.
end

% Note that for IT, a long time vector will be generated in start_calc,
% which is needed to catch the maximum dose metric over time during pulsed
% exposure.
if glo.sel == 2 % for individual tolerance ...
    DM = Xout(:,glo.locDM);  % take the correct state variable as dose metric
    S = Xout(:,glo.locS);    % take background survival from the model
    % DM(DM<=0) = 1e-20;     % take care of internal conc <= 0 (probably not needed)
    z = max(z,1e-20);        % make sure that the NEC is not zero ...
    maxDM = zeros(size(DM)); % intitialise a new vector for maximum DM over time
    for j = 1:length(t),     % make sure that DM does not decrease in time (dead animals dont become alive)
        maxDM(j,:) = max(DM(1:j,:)); % so look for max of DM over time
    end
    if glo.dis == 1 % use log-logistic distribution
        S = (1 ./ (1+(maxDM/z).^beta)) .* S; % survival probability
        % the survival due to the chemical is multiplied with the background
        % survival (calculated in derivatives), assuming logistic distrib.
    elseif glo.dis == 2 % use log-normal distribution
        S = (1-normcdf(log10(maxDM),log10(z),log10(Fs)/1.96))  .* S; % survival probability
        % the survival due to the chemical is multiplied with the background
        % survival (calculated in derivatives), assuming log-normal distrib.
        % this requires the statistics toolbox.
    end
    Xout(:,glo.locS) = S; % replace correct state by newly calculated survival prob.
end

% =========================================================================
%                   EVENTS FUNCTION, MODIFY IF EVENTS ARE ON
% =========================================================================

function [value,isterminal,direction] = eventsfun(t,X,par,c)

% This subfunction catches the `events`: in this case, it looks for
% exceedance of the internal threshold concentration.
%
% Note that the eventsfun has the same inputs, in the same sequence, as
% derivatives.m

global glo

z       = par.z(1);  % threshold internal concentration
nevents = 1;         % number of events that we try to catch

value      = zeros(nevents,1); % initialise with zeros
value(1)   = X(glo.locDM) - z; % thing to follow is dose metric minus threshold
isterminal = zeros(nevents,1); % do NOT stop the solver at an event
direction  = zeros(nevents,1); % catch ALL zero crossing when function is increasing or decreasing
